/*******************************************************************************
 * Copyright (c) 2004 IBM Corporation.
 * All rights reserved. 
 *******************************************************************************/
package com.ibm.gallery.common.wizards;

import java.text.MessageFormat;

import org.eclipse.help.ILiveHelpAction;
import org.eclipse.jface.wizard.IWizard;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import com.ibm.ccl.ua.wizards.*;



/**
 * Abstract Action Delegate.
 */
public abstract class GalleryWizardAction implements ILiveHelpAction, GalleryConstants {
	protected String wizID;
	public abstract IWizard getWizard();
	public void initWizard(IWizard wizard){
		// can add init code
	}

	/* (non-Javadoc)
	 * @see org.eclipse.help.ILiveHelpAction#setInitializationString(java.lang.String)
	 * 
	 * handle the arg parameter passed from the javascript link
	 */
	public void setInitializationString(String wiz) {
		// save the wizard ID, it is used by the ImportAction class getWizard()
		wizID = wiz;
	}
	
	/* (non-Javadoc) 
	 * Runs the live help action.
	 * @see java.lang.Runnable#run()
	 */
	public void run() {
		// This method will be called by the help system from a non-UI thread,
		// so UI access must be wrapped in a Display.syncExec() method.
		RunnableWizardDelegate runnableWizard = new RunnableWizardDelegate(this);
		Display.getDefault().syncExec(runnableWizard);
	}
	
	private class RunnableWizardDelegate implements Runnable {
		private GalleryWizardAction actionDelegate;
		public RunnableWizardDelegate(GalleryWizardAction delegate) {
			actionDelegate = delegate;
		}
		
		/* (non-Javadoc)
		 * @see java.lang.Runnable#run()
		 */
		public void run() {
			// get the wizard
			IWizard wizard= actionDelegate.getWizard();
			if(wizard == null) {
				// No wizard found.
				// No wizard returned from getWizard() in Action Delegate Class: {0}
				String actionDelegateClassName = actionDelegate.getClass().getName();
				String message = MessageFormat.format(I18N.ERROR_GALLERY_WIZACTION_NO_WIZARD, new Object[] {actionDelegateClassName});
				Activator.logWarning(message); 
			}
			try {
				IWorkbenchWindow window = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
				if (window != null && wizard != null) {
					actionDelegate.initWizard(wizard);
					// Bring the Workbench window to the top of other windows.
					// On some Windows systems, the Workbench icon will flash on the task bar 
					Shell shell = window.getShell();
					shell.setMinimized(false);
					shell.forceActive();

					WizardDialog dialog = new WizardDialog(window.getShell(), wizard);
					dialog.create();
					dialog.open();
				}
			} catch (Exception e){
				// catch errors like bad zip file name
				Activator.logError("Exception in GalleryWizardAction: ", e); //$NON-NLS-1$
				Activator.logWarning("wizard = "+wizard); //$NON-NLS-1$
				e.printStackTrace();
			}	
			
		}
		
	}
}