/*******************************************************************************
 * Copyright (c) 2004, 2006 IBM Corporation.
 * All rights reserved. 
 *******************************************************************************/
package com.ibm.gallery.common.wizards;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.text.MessageFormat;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Preferences;
import org.eclipse.help.ITocContribution;
import org.eclipse.help.internal.base.HelpBasePlugin;
import org.eclipse.help.internal.base.IHelpBaseConstants;
import org.eclipse.help.internal.base.remote.PreferenceFileHandler;
import org.eclipse.help.internal.base.remote.RemoteHelp;
import org.eclipse.help.internal.dynamic.DocumentReader;
import org.eclipse.jface.wizard.IWizard;
import org.eclipse.jface.wizard.Wizard;

import com.ibm.ccl.ua.wizards.*;


public class GalleryWizardActionHelper implements GalleryConstants {
	public static final String ELEM_WIZARD = "wizard"; //$NON-NLS-1$
	public static final String GALLERY_ID = "com.ibm.ccl.ua.wizards"; //$NON-NLS-1$
	public static final String GALLERY_IMPORT_WIZARD = "importWizard"; //$NON-NLS-1$
		
	//Added for handling Samples support
	public static IWizard getSampleGalleryImportWizard(String extensionId) {
		return getWizardWithExtensionId(extensionId, GALLERY_ID, GALLERY_IMPORT_WIZARD);
	}
	
	
	public static IWizard getWizardWithExtensionId(String extensionId) {
		return getWizardWithExtensionId(extensionId, GALLERY_ID, GALLERY_IMPORT_WIZARD);
	}
	
	public static IWizard getWizardWithExtensionId(String extensionId, String namespace, String extensionPointName) {
		IWizard wizard = null;
		// get the extension point that the wizard we want to use extends
		IExtensionPoint xpt = GalleryWizardActionHelper.getExtensionPoint(namespace, extensionPointName);	
		if(null == xpt) {
			// Extension point {0} not found in namespace {1}
			
			
				String message = MessageFormat.format(I18N.ERROR_GALLERY_WIZHELP_EXTENSIONPOINT_NOT_FOUND, new Object[] {extensionPointName, namespace});
				Activator.logWarning(message);
				return null;
			
		}
		IExtension extension = xpt.getExtension(extensionId);
		if (extension != null) {
			// find the correct ConfigurationElement
			IConfigurationElement configElement = null;
			IConfigurationElement[] configElements = extension.getConfigurationElements();
			for (int i = 0; i < configElements.length; i++) {
				if (configElements[i].getName().equals(ELEM_WIZARD)) { //$NON-NLS-1$
					configElement = configElements[i];
					break;
				}
			}
			if(null == configElement) {
				// Configuration Element {0} not found. Extension ID: {1} , Exension point: {2} , Namespace: {3}
				String message = MessageFormat.format(I18N.ERROR_GALLERY_WIZHELP_CONFIG_ELEM_NOT_FOUND, new Object[] {ELEM_WIZARD, extensionId, extensionPointName, namespace});
				Activator.logWarning(message);
				return null;
			}
			// create the wizard
			try {
				//wizard = (IWizard) configElement.createExecutableExtension("class");
				wizard = (Wizard) configElement.createExecutableExtension("class");
				// v601 code
				// wizard = (IWizard) configElement.createExecutableExtension(WizardsRegistryReader.UNCATEGORIZED_WIZARD_CATEGORY);
			} catch (CoreException e) {
				// Error creating executable extension. Extension ID: {0} , Configuration element name: {1} , Extension point {2}, Namespace: {3}
				String message = MessageFormat.format(I18N.ERROR_GALLERY_WIZHELP_CREATE_EXECUTABLE_EXTENSION, new Object[] {extensionId, ELEM_WIZARD, extensionPointName, namespace});
			Activator.logError(message, e);
				return null;
			}
		} else {
			
			// Extension id {0} not found for extension point {1} in namespace {2}
			//Check if remote assistance is enabled.  If so, search all registered ICs for the plugin in the extension.
			Preferences prefs = HelpBasePlugin.getDefault().getPluginPreferences();
			String isRemoteAssistanceEnabled= prefs.getString(IHelpBaseConstants.P_KEY_REMOTE_HELP_ON);
			
			if(isRemoteAssistanceEnabled.equals("true"))
			{
				//Get plugin id, then search all registered ICs for that plugin
				//GET PLUGIN ID
				
				PreferenceFileHandler prefHandler = new PreferenceFileHandler();
				String host[] = prefHandler.getHostEntries();
				String port[] = prefHandler.getPortEntries();
				String path[] = prefHandler.getPathEntries();
				String isEnabled[] = prefHandler.isEnabled();

				int numICs = host.length;
				if (numICs == 0) // No remote InfoCenters in preferences.ini so no wizard found
					return null;

				InputStream in = null;
				URL url = null;
				String urlStr = "", wizardPathString="", urlString; //$NON-NLS-1$
				for (int i = 0; i < numICs; i++) {
					if (isEnabled[i].equalsIgnoreCase("true")) { //$NON-NLS-1$
						try {
							if(path[i].endsWith("/"))
								wizardPathString=path[i] + "mapper/wizardBuilder" + '?' + "wizardID" + '=' + extensionId+"&namespace="+namespace+"&extensionPointName="+extensionPointName;
							else
								wizardPathString=path[i] + "/mapper/wizardBuilder" + '?' + "wizardID" + '=' + extensionId+"&namespace="+namespace+"&extensionPointName="+extensionPointName;
							
							urlString="http://"+ host[i]+":"+new Integer(port[i]).intValue()+ wizardPathString; //$NON-NLS-1$
							
							url = new URL(urlString);
							in = url.openStream();

							if (in != null) { //Found wizard extension on a remote ic
								
								int count = 0, initialBytes=in.available();
								byte[] buf = new byte[initialBytes];
																
								while(count!=initialBytes){
									in.read(buf);
									count=count+1;
								}
								
								String wizardInfoString=new String(buf);
								String [] wizardInfoArray=wizardInfoString.split(",");
								String wizardURL="http://"+ host[i]+":"+new Integer(port[i]).intValue()+path[i];
								
								ExampleProjectCreationWizard importWizard = new ExampleProjectCreationWizard();
								importWizard.addNewPage(new ExampleProjectCreationWizardPage(0,wizardInfoArray[0],wizardInfoArray[6],wizardInfoArray[1],wizardInfoArray[2],wizardInfoArray[6],wizardInfoArray[5],wizardInfoArray[7],wizardInfoArray[8],wizardInfoArray[9],wizardInfoArray[10],wizardURL));
								
								wizard=importWizard;
								
							}
						} catch (Throwable t) {
					       t.printStackTrace();
						} finally {
							if (in != null) {
								try {
									in.close();
									in = null;
								} catch (IOException e) {
									// nothing more we can do
								}
							}
						}
					}
				}
				
				
			}
			else
			{
				String message = MessageFormat.format(I18N.ERROR_GALLERY_WIZHELP_EXTENSIONID_NOT_FOUND, new Object[] {extensionId, extensionPointName, namespace});
				Activator.logWarning(message);
			}
			
		}
		
		return wizard;
	}
	//
	// looks for the wizard defined in the plugin extensions
	//
	public static IWizard findWizard(String attributeName, String className, String namespace, String extensionPointName) {
		IWizard wizard = null;
		// get the extension point that the wizard extends
		IExtensionPoint xpt = getExtensionPoint(namespace, extensionPointName);
		IConfigurationElement configElement = null;
		IExtension[] extensions = xpt.getExtensions();
		for (int i = 0; i < extensions.length; i++) {
			IExtension ext = extensions[i];
			IConfigurationElement[] configElements = ext.getConfigurationElements();
			for (int k = 0; k < configElements.length; k++) {
				if (configElements[k].getName().equals(ELEM_WIZARD)) {
					String id = configElements[k].getAttribute(attributeName);
					if (id != null) {
						if (id.equals(className)) {
							configElement = configElements[k];
							break;
						}
					}
				}
			}
			if (configElement != null) {
				break;
			}
		}
		try {
			// changed ATT_CLASS to BASE_CATEGORY in v601 for elcipse 3.1
			// changed to "class" for v7
			wizard = (IWizard) configElement.createExecutableExtension("class");
		} catch (CoreException e) {
			Activator.logError("GalleryWizardActionHelper findWizard() ", e);
			e.printStackTrace();
		}
		return wizard;
	}
	private static IExtensionPoint getExtensionPoint(String namespace, String extensionPointName) {
		return Platform.getExtensionRegistry().getExtensionPoint(namespace, extensionPointName); 
	}
	
	
}
